@@ -24,10 +24,13 @@
 #include <config.h>
 #endif
 
-#include <ell/ell.h>
+#define _GNU_SOURCE
+#include <stdio.h>
+#include <signal.h>
+
 #include <readline/history.h>
 #include <readline/readline.h>
-#include <stdio.h>
+#include <ell/ell.h>
 
 #include "agent.h"
 #include "command.h"
@@ -165,8 +168,9 @@ void display_refresh_set_cmd(const char *family, const char *entity,
 		display_refresh.argc = argc;
 
 		display_refresh.argv = l_new(char *, argc + 1);
+
 		for (i = 0; i < argc; i++)
-			display_refresh.argv[i] = argv[i];
+			display_refresh.argv[i] = l_strdup(argv[i]);
 
 		l_queue_clear(display_refresh.redo_entries, l_free);
 
@@ -322,14 +326,14 @@ void display_table_footer(void)
 void display_command_line(const char *command_family,
 						const struct command *cmd)
 {
-	char *cmd_line = l_strdup_printf("%s%s%s%s%s %s",
+	char *cmd_line = l_strdup_printf("%s%s%s%s%s%s%s",
 				command_family ? : "",
 				command_family ? " " : "",
 				cmd->entity ? : "",
 				cmd->entity  ? " " : "",
 				cmd->cmd,
-				cmd->arg ? : "",
-				cmd->arg ? " " : "");
+				cmd->arg ? " " : "",
+				cmd->arg ? : "");
 
 	display(MARGIN "%-*s%s\n", 50, cmd_line, cmd->desc ? : "");
 
@@ -577,29 +581,19 @@ void display_quit(void)
 	rl_crlf();
 }
 
-static void signal_handler(struct l_signal *signal, uint32_t signo,
-								void *user_data)
+static void signal_handler(void *user_data)
 {
-	switch (signo) {
-	case SIGWINCH:
-		if (display_refresh.cmd)
-			display_refresh_reset();
-		break;
-	}
+	if (display_refresh.cmd)
+		display_refresh_reset();
 }
 
 void display_init(void)
 {
-	sigset_t mask;
-
 	display_refresh.redo_entries = l_queue_new();
 
 	setlinebuf(stdout);
 
-	sigemptyset(&mask);
-	sigaddset(&mask, SIGWINCH);
-
-	resize_signal = l_signal_create(&mask, signal_handler, NULL, NULL);
+	resize_signal = l_signal_create(SIGWINCH, signal_handler, NULL, NULL);
 
 	rl_attempted_completion_function = command_completion;
 	rl_completion_display_matches_hook = display_completion_matches;
