@@ -27,7 +27,7 @@
 #include <netinet/ip.h>
 #include <net/ethernet.h>
 #include <linux/types.h>
-#include <linux/if_arp.h>
+#include <net/if_arp.h>
 #include <errno.h>
 #include <time.h>
 
@@ -935,7 +935,7 @@ bool _dhcp_client_set_transport(struct l_dhcp_client *client,
 		return false;
 
 	if (client->transport)
-		_dhcp_transport_free(transport);
+		_dhcp_transport_free(client->transport);
 
 	client->transport = transport;
 	return true;
