@@ -24,9 +24,11 @@
 #include <config.h>
 #endif
 
+#define _GNU_SOURCE
 #include <stdlib.h>
 #include <stdio.h>
 #include <errno.h>
+#include <time.h>
 #include <sys/socket.h>
 #include <limits.h>
 #include <linux/if.h>
@@ -254,8 +256,15 @@ static void scan_triggered(struct l_genl_msg *msg, void *userdata)
 		sr->passive ? "Passive" : "Active", sc->ifindex);
 	sr->triggered = true;
 
-	if (sr->trigger)
+	if (sr->trigger) {
 		sr->trigger(0, sr->userdata);
+
+        /*
+         * Reset callback for the consequent scan triggerings of the
+         * multi-segmented scans.
+         */
+        sr->trigger = NULL;
+    }
 }
 
 struct scan_freq_append_data {
@@ -785,7 +794,8 @@ static bool scan_parse_bss_information_elements(struct scan_bss *bss,
 			if (ie_parse_supported_rates(&iter,
 						&bss->supported_rates) < 0)
 				l_warn("Unable to parse [Extended] "
-					"Supported Rates IE");
+					"Supported Rates IE for "
+					MAC, MAC_STR(bss->addr));
 			break;
 		case IE_TYPE_RSN:
 			if (!bss->rsne)
@@ -795,7 +805,8 @@ static bool scan_parse_bss_information_elements(struct scan_bss *bss,
 		case IE_TYPE_BSS_LOAD:
 			if (ie_parse_bss_load(&iter, NULL, &bss->utilization,
 						NULL) < 0)
-				l_warn("Unable to parse BSS Load IE");
+				l_warn("Unable to parse BSS Load IE for "
+					MAC, MAC_STR(bss->addr));
 			else
 				l_debug("Load: %u/255", bss->utilization);
 
